@@ -98,6 +98,12 @@ print_device_info(drmDevicePtr device, int i, bool print_revision)
             printf("                    %s\n", *compatible);
             compatible++;
         }
+    } else if (device->bustype == DRM_BUS_FAUX) {
+        printf("|   +-> faux\n");
+
+        printf("+-> deviceinfo\n");
+        printf("    +-> faux\n");
+        printf("        +-> name  %s\n", device->deviceinfo.faux->name);
     } else {
         printf("Unknown/unhandled bustype\n");
     }
