@@ -843,6 +843,7 @@ extern char *drmGetRenderDeviceNameFromFd(int fd);
 #define DRM_BUS_USB       1
 #define DRM_BUS_PLATFORM  2
 #define DRM_BUS_HOST1X    3
+#define DRM_BUS_FAUX      4
 
 typedef struct _drmPciBusInfo {
     uint16_t domain;
@@ -889,6 +890,10 @@ typedef struct _drmHost1xDeviceInfo {
     char **compatible; /* NULL terminated list of compatible strings */
 } drmHost1xDeviceInfo, *drmHost1xDeviceInfoPtr;
 
+typedef struct _drmFauxDeviceInfo {
+    char *name;
+} drmFauxDeviceInfo, *drmFauxDeviceInfoPtr;
+
 typedef struct _drmDevice {
     char **nodes; /* DRM_NODE_MAX sized array */
     int available_nodes; /* DRM_NODE_* bitmask */
@@ -904,6 +909,7 @@ typedef struct _drmDevice {
         drmUsbDeviceInfoPtr usb;
         drmPlatformDeviceInfoPtr platform;
         drmHost1xDeviceInfoPtr host1x;
+        drmFauxDeviceInfoPtr faux;
     } deviceinfo;
 } drmDevice, *drmDevicePtr;
 
@@ -926,6 +932,11 @@ extern int drmGetDeviceFromDevId(dev_t dev_id, uint32_t flags, drmDevicePtr *dev
  */
 extern int drmGetNodeTypeFromDevId(dev_t devid);
 
+/**
+ * Check if two drmDevice pointers represent the same DRM device.
+ *
+ * Returns 1 if the devices are equal, 0 otherwise.
+ */
 extern int drmDevicesEqual(drmDevicePtr a, drmDevicePtr b);
 
 extern int drmSyncobjCreate(int fd, uint32_t flags, uint32_t *handle);
